{-# LANGUAGE QuasiQuotes, OverloadedStrings, FlexibleContexts, UnicodeSyntax,
             EmptyDataDecls, ScopedTypeVariables,
             GeneralizedNewtypeDeriving, FlexibleInstances #-}
{-# OPTIONS_GHC -F -pgmF frquotes #-}
{-# OPTIONS -Wall -fno-warn-missing-signatures #-}

import Language.LaTeX
import qualified Language.LaTeX.Builder as B
import qualified Language.LaTeX.Builder.Internal as BI
import qualified Language.LaTeX.Builder.Beamer as BM
import qualified Language.LaTeX.Builder.Math as M
import qualified Language.LaTeX.Length as L
import Language.LaTeX.Slicer
import Language.LaTeX.Builder.QQ
import Control.Monad.Writer
import Control.Applicative
import Control.Arrow hiding ((<+>))
import Data.String
import Data.Monoid.Unicode

todo :: a -> a
todo = id
{-# DEPRECATED todo "You have something to do here" #-}


docName = "fresh-look"

ø :: Monoid m => m
ø = (Data.Monoid.Unicode.∅)

-- this option should be used to print only the document body and not the
-- documentclass nor the preamble
withFmt = False -- True

main = quickView myViewOpts{basedir="out",pdflatex=pdflatex',showoutput=False,pdfviewer="echo"} docName doc
  where pdflatex' | withFmt   = "tex '&latex'"
                  | otherwise = pdflatex myViewOpts

doc = B.document dc preamb body where
  dc = BM.documentclass [BM.t,BM.red,BM.compress]

usepackages = mconcat . map (BI.usepackage [] . BI.pkgName)

preamb = ø
       ⊕ usepackages ["amsmath","amssymb"]
       ⊕ BI.usepackage ["T3","OT1"] (BI.pkgName "fontenc")
      -- These two declarations are from the tipa package.
      -- However importing the packages breaks the document.
      -- Note that we require T3 fonts here.
       ⊕ BI.rawPreamble [$str|\DeclareTextSymbol\textcrlambda{T3}{172}|]
       ⊕ BI.rawPreamble [$str|\DeclareTextSymbolDefault\textcrlambda{T3}|]
       ⊕ usepackages ["ucs"]
       ⊕ BI.usepackage ["utf8x"] (BI.pkgName "inputenc")
       ⊕ usepackages ["rotating","tikz"]
       ⊕ BI.rawPreamble [$str|\usetikzlibrary{matrix,arrows}|]
       ⊕ B.title «A fresh look at programming with names and binders»
          -- 'and' improve
       ⊕ B.author «Nicolas Pouillard and François Pottier»
       ⊕ B.institute (B.texttt "{Nicolas.Pouillard,Francois.Pottier}@inria.fr")
       ⊕ BI.rawPreamble "\\date{Gallium Seminar, May 2010}"
       ⊕ BM.beamertemplatenavigationsymbolsempty
       ⊕ BM.useoutertheme [] "default"
       ⊕ BM.useinnertheme [("shadow","true")] "rounded"
       ⊕ BM.usecolortheme [] "orchid"
       ⊕ footline
       ⊕ margins
       ⊕ unicodesymbols

footline :: PreambleItem
       -- ⊕ BM.footline BM.defaultFootline{BM.showTotalFrames=False}
footline =
  BI.rawPreamble $
  [$istr|\defbeamertemplate*{footline}{}
        |{
        |  \leavevmode%
        |  \hbox{%
        |    \begin{beamercolorbox}[wd=\paperwidth,ht=2.25ex,dp=1.125ex,right]{}%
        |      \insertframenumber{}\hspace*{2ex}
        |    \end{beamercolorbox}}%
        |  \vskip0pt%
        |}
        |]

unicodesymbols = BI.rawPreamble [$str|
\providecommand\mathbbm{\mathbb}
\DeclareUnicodeCharacter{737}{\textsuperscript{l}}
\DeclareUnicodeCharacter{8718}{\ensuremath{\blacksquare}}
\DeclareUnicodeCharacter{8759}{::}
\DeclareUnicodeCharacter{9669}{\ensuremath{\triangleleft}}
\DeclareUnicodeCharacter{8799}{\ensuremath{\stackrel{\scriptscriptstyle ?}{=}}}
\DeclareUnicodeCharacter{10214}{\ensuremath{\llbracket}}
\DeclareUnicodeCharacter{10215}{\ensuremath{\rrbracket}}

\DeclareUnicodeCharacter{231}{\c{c}}                            % ç
\DeclareUnicodeCharacter{411}{\textcrlambda}                    % ƛ
\DeclareUnicodeCharacter{945}{\ensuremath{\alpha}}              % α
\DeclareUnicodeCharacter{946}{\ensuremath{\beta}}               % β
\DeclareUnicodeCharacter{947}{\ensuremath{\gamma}}              % γ
\DeclareUnicodeCharacter{948}{\ensuremath{\delta}}              % δ
\DeclareUnicodeCharacter{949}{\ensuremath{\epsilon}}            % ε
\DeclareUnicodeCharacter{953}{\ensuremath{\iota}}               % ι
\DeclareUnicodeCharacter{954}{\ensuremath{\kappa}}              % κ
\DeclareUnicodeCharacter{955}{\ensuremath{\lambda}}             % λ
\DeclareUnicodeCharacter{957}{\ensuremath{\nu}}                 % ν
\DeclareUnicodeCharacter{959}{\ensuremath{o}}                   % ο
\DeclareUnicodeCharacter{963}{\ensuremath{\sigma}}              % σ
\DeclareUnicodeCharacter{964}{\ensuremath{\tau}}                % τ
\DeclareUnicodeCharacter{969}{\ensuremath{\omega}}              % ω
\DeclareUnicodeCharacter{915}{\ensuremath{\Gamma}}              % Γ
\DeclareUnicodeCharacter{916}{\ensuremath{\Delta}}              % Δ
\DeclareUnicodeCharacter{183}{\ensuremath{\cdot}}               % ·
\DeclareUnicodeCharacter{8605}{\ensuremath{\leadsto}}           % ↝
\DeclareUnicodeCharacter{8614}{\ensuremath{\mapsto}}            % ↦
\DeclareUnicodeCharacter{8636}{\ensuremath{\leftharpoonup}}     % ↼
\DeclareUnicodeCharacter{8658}{\ensuremath{\Rightarrow}}        % ⇒
\DeclareUnicodeCharacter{10230}{\ensuremath{\longrightarrow}}   % ⟶
\DeclareUnicodeCharacter{9733}{\ensuremath{\star}}              % ★
\DeclareUnicodeCharacter{9659}{\ensuremath{\vartriangleright}}  % ▻
%\DeclareUnicodeCharacter{9669}{\ensuremath{\vartriangleleft}}  % ◅

\newcommand\defeq{\stackrel{\text{def}}{=}}
\DeclareUnicodeCharacter{8797}{\ensuremath{\defeq}}             % ≝
\newcommand\textLambda{\ensuremath{\Lambda}}
|]

p = B.para

vcenter x = B.vfill ⊕ x ⊕ B.vfill

slide title = put . BM.slide title . mapNonEmpty vcenter . execWriter

slideL title lbl = BM.frame [] [] [BM.label lbl] title mempty

put :: ParItem -> ParItemW
put = tell

ovFromInt = BM.ovFrom . BM.ovInt

alt i x y = BM.only (BM.ovInts [i]) x ⊕ BM.only [ovFromInt (i+1)] y

altI = BM.alt . BM.ovInts

alts i xs = (foldr (.) id . zipWith (altI . pure) [i..] $ init xs) $ last xs

temporalL ov [x,y,z] = BM.temporal ov x y z
temporalL _  _       = error "temporalL"
temporalLI = temporalL . BM.ovInts . pure
temporalI = BM.temporal . BM.ovInts . pure

margins = BM.setbeamersize (BM.TextMarginLeft (L.cm 0.3))
        ⊕ BM.setbeamersize (BM.TextMarginRight (L.cm 0.3))

bfHuge   = B.decls [B._Huge, B.bfseries]
bfLarge  = B.decls [B._Large, B.bfseries]
ttHuge   = B.decls [B._Huge, B.ttfamily]
--tthuge   = B.decls [B.huge, B.ttfamily]
--ttlarge  = B.decls [B.large, B.ttfamily]
--ttLarge  = B.decls [B._Large, B.ttfamily]
--ttLARGE  = B.decls [B._LARGE, B.ttfamily]

slideCs title subtitle =
    slide ø . put . B.center . (⊕subt) . B.para $ title
  where subt = mapNonEmpty ((B.vfill ⊕) . B.para) subtitle

slideC x = slideCs x ø

slideCB = slideC . bfHuge

slideO title = put . BM.slideO title BM.ovIncr . vcenter . execWriter

mathlift f = M.text . f . ma

onlyI = BM.only . BM.ovInts

onlyenvI :: [Int] -> ParItem -> ParItem
onlyenvI = BM.onlyenv . BM.ovInts

visibleenvI :: [Int] -> ParItem -> ParItem
visibleenvI = BM.visibleenv . BM.ovInts

visibleI = BM.visible . BM.ovInts
invisibleI = BM.invisible . BM.ovInts

visibleFromInt = BM.visible . pure . ovFromInt
visibleIncr = BM.visible BM.ovIncr

box = BM.block

myMverb = B.texttt . B.protector (myXchar (M.mchar B.ttchar))  -- M.verb

--myXchar xchar '·'  = B.makebox (L.ex 1) Centered (BI.rawTex "$\\cdotp$") --(xchar '·')
--myXchar xchar '·'  = B.makebox (L.ex 1) Centered (xchar '·')
myXchar xchar x
  | x `elem` "·ƛ_" = B.makebox (L.ex 1.22) Centered (xchar x)
  | x `elem` "="   = B.ttchar x
  | x `elem` "{}"  = M.mchar B.hchar x
myXchar xchar x    = xchar x

hs = myMverb . dropWhile (=='\n')

hsLarge = B.decl B._Large . hs
hsLARGE = B.decl B._LARGE . hs
hshuge  = B.decl B.huge . hs
hsHuge  = B.decl B._Huge . hs

paraC = put . B.center . p

alertC = paraC . BM.alert

example :: LatexItem -> ParItemW
example = put . box ø . B.para

exampleHS :: String -> ParItemW
exampleHS = example . hs

exampleHS' :: String -> ParItemW
exampleHS' = put . B.para . hs

slideHS title = slide title . exampleHS

altHS = put . box ø . B.para . mconcat . zipWith (BM.only . BM.ovInts . pure) [1..] . map hs

itemize = B.itemize
itemizeO = BM.itemize BM.ovIncr

description = B.description
descriptionO = BM.description BM.ovIncr

item = tell . pure . B.item . B.para

ma = B.math
mh :: HS a -> LatexItem
mh = B.math . unHS

class App repr where
  (·) :: repr (a -> b) -> repr a -> repr b

class Cond repr where
  cond :: repr Bool -> repr a -> repr a -> repr a

newtype HS a = HS { unHS :: MathItem }
  deriving (Eq,Show,Num,IsString,Monoid)

instance App HS where
  -- HS f · HS x = HS (M.parens (f ⊕ M.thickspace ⊕ x))
  HS f · HS x = HS (f ⊕ M.thickspace ⊕ x)

instance Cond HS where
  -- bold the keywords
  cond (HS b) (HS t) (HS e)
    = HS ("if "⊕b⊕" then "⊕t⊕" else "⊕e)

    -- do we want a type constraint here?
{-
instance IsString (HS a) where fromString = HS . fromString

instance Monoid (HS a) where
  mempty = HS mempty
  HS x `mappend` HS y = HS (x `mappend` y)
-}

(<+>) :: HS a -> HS b -> HS c -- Monoid a => a -> a -> a
HS x <+> HS y = HS (x ⊕ M.thickspace ⊕ y)

data KSet
type KStar = KSet
data KWorld
data SKind
data Name
data Binder
data BINDER
data Incl
class HSRel r
instance HSRel KSet
instance HSRel KWorld
class HSType a
instance HSType Integer
instance (HSType a, HSType b) => HSType (a -> b)
instance (HSType a) => HSType (Maybe a)
instance (HSType a) => HSType [a]
instance HSType ()

-- TODO
agdabin op = "_" ⊕ op ⊕ "_"
binop :: HS (a -> b -> c) -> HS a -> HS b -> HS c
binop op x y = x <+> op <+> y
lam x e = hs0"λ "⊕x⊕hs0". "⊕e
binoppack :: HS (a -> b -> c) -> (HS (a -> b -> c), (HS a -> HS b -> HS c))
binoppack op = (agdabin op, binop op)
operatorname x y = x ⊕ parens y
mathord = mempty
idxrefl :: IsString s => (a -> a -> b) -> (s -> a) -> b
idxrefl rel pat = rel (pat "₁") (pat "₂")
--logeq :: (HSRel r, HSType a) => HS a -> HS a -> HS r -> HS Bool
logeq x y z = x <+> ("(" ⊕ z ⊕ ")") <+> y
--rlogeq :: (HSRel r, HSType a) => HS a -> HS r -> HS Bool
rlogeq k = idxrefl logeq (const k)
--ilogeq :: (HSRel r, HSType a) => HS a -> HS r -> HS Bool
ilogeq txt = idxrefl logeq (txt ⊕)
hs0 x = HS (M.text (hs x))
_World :: HS SKind
_World = hs0 "World"
_Name :: HS (KWorld -> KStar)
_Name = hs0 "Name"
_Maybe :: HS (KStar -> KStar)
_Maybe = hs0 "Maybe"
_Fin :: HS (Integer -> KStar)
_Fin = hs0 "Fin"
nothing :: HS (Maybe a)
nothing = hs0 "nothing"
just :: HS (a -> Maybe a)
just = hs0 "just"
suc :: HS (Integer -> Integer)
suc = hs0 "suc"
unit :: HS ()
unit = hs0"()"
_List :: HS (KStar -> KStar)
_List = hs0 "List"
_Tm :: HS (KWorld -> KStar)
_Tm = hs0 "Tm"
_Binder :: HS (KWorld -> KWorld -> KStar)
_Binder = hs0 "Binder"
_BINDER :: HS (KWorld -> KWorld -> KStar)
_BINDER = hs0 "BINDER"
-- The set of all atoms.
atomset = M.mathbb "A"
-- Fresh for.
(ff_,ff) = binoppack "\\#"
-- Abstractions.
-- World overriding.
wover x y z = "("⊕x⊕","⊕y⊕") ⊞ "⊕z
rwover = idxrefl wover
iwover txt = rwover (txt ⊕)
wshift :: HS KWorld -> HS KWorld
wshift (HS x) = HS (x ⊕ mathord ⊕ M.uparrow)
wshiftn (HS x) (HS y) = HS (y ⊕ mathord ⊕ M.uparrow ⊕ M.sup x)
class Greek a where
  -- greekChar :: Char -> a
  α :: a
  β :: a
  τ :: a
  σ :: a

instance Greek MathItem where
  α = M.alpha
  β = M.beta
  τ = M.tau
  σ = M.sigma

instance Greek (HS a) where
  α = HS M.alpha
  β = HS M.beta
  τ = HS M.tau
  σ = HS M.sigma

instance Greek (LatexM LatexItm) where
  α = ma M.alpha
  β = ma M.beta
  τ = ma M.tau
  σ = ma M.sigma

sub :: HS a -> HS a
sub = HS . M.sub . unHS
-- Here we use a long double right arrow
-- to represent a simple double right arrow.
(⟹) :: HS a -> HS b -> HS c
-- impl = agdabin "⇒"
(⟹) = binop "⇒"
-- Here we use a long right arrow
-- to represent a simple right arrow.
(⟶) = binop "→"
(×) :: HS KStar -> HS KStar -> HS KStar
(×) = binop "×"
(∧) :: HS Bool -> HS Bool -> HS Bool
(∧) = binop "∧"
(⊔) :: HS Integer -> HS Integer -> HS Integer
(max_,(⊔)) = binoppack "⊔"
_ℕ :: HS KSet
_ℕ = "ℕ"
_Pred :: HS (KSet -> KSet)
_Pred = hs0 "Pred"
(eqName_, eqName) = binoppack ("≟"⊕sub (hs0 "Name"))
eqNat_ :: HS (Integer -> Integer -> Bool)
eqNat :: HS Integer -> HS Integer -> HS Bool
(eqNat_, eqNat) = binoppack ("≟"⊕sub "ℕ")
(eq, (≡)) = binoppack "≡"
(incl, (⊆)) = binoppack "⊆"
(mem, (∈)) = binoppack "∈"
(union, (∪)) = binoppack "∪"
(eqdef, (≝)) = binoppack "≝"
dom = operatorname (hs0 "dom")
codom = operatorname (hs0 "codom")
parens :: HS a -> HS a
parens (HS x) = HS (M.parens x)
importID :: HS (Integer -> Integer -> Integer)
importID = hs0 "import⊆"
importIN :: HS (() -> Integer -> Integer)
importIN = hs0 "import⊆"
transID :: HS (Integer -> Integer -> Integer)
transID = hs0 "⊆-trans"
transIN :: HS (() -> () -> ())
transIN = hs0 "⊆-trans"
reflI = hs0 "⊆-refl"
nameOfBinderN :: HS (Integer -> Integer)
nameOfBinderN = hs0 "nameOfBinder"
nameOfBinderD_ :: HS (() -> Integer)
nameOfBinderD_ = hs0 "nameOfBinder"
nameOfBinderD :: () -> HS Integer
nameOfBinderD () = nameOfBinderD_ · unit
exportBinderN :: HS (Integer -> Integer -> Maybe Integer)
exportBinderN = hs0 "exportBinder"
exportBinderD :: () -> HS Integer -> HS (Maybe Integer)
exportBinderD_ = HS "exportBinder"
exportBinderD () x = exportBinderD_ · unit · x
nextBINDERN :: HS (Integer -> Integer -> Integer)
nextBINDERN = hs0 "nextBINDER"
nextBINDERD :: HS (() -> () -> ())
nextBINDERD = hs0 "nextBINDER"
weaken = hs0 "weaken"
dropNameD :: HS (() -> Integer)
dropNameD = hs0 "dropName"
dropNameN :: HS (Integer -> ())
dropNameN = hs0 "dropName"
comBinderIncl :: HS (Binder -> Incl -> (Incl, Binder))
comBinderIncl = hs0 "Binder-commute-⊆"

-- style this
freshml = B.textsc «FreshML»
-- alphaCaml = «C{α}ml»
purefreshml = «pure {freshml}»
-- ott = B.textsc «Ott»

body = slice . execWriter $ do
  put B.maketitle

  slideCB «Towards safer and more expressive languages for meta-programming»
  -- that's the direction we took with François for my Ph.D
  -- why two goals at once? because they are complementary...
  -- by meta-programming we means all programs that manipulate some kind of programs

  slideCB «Program representation should stay well-typed and {BM.alert «well-scoped»}»
  -- the well-typed part guarantee that we can only build well-typed program
  --   representation, the benefit counter part is that you learn part of the
  --   type when pattern matching a program
  -- here we will omit the "well-typed" part since this talk is dedicated to
  --   bindings and scopes
  -- the well-scoped part guarantee that we don't generate programs with
  --   "unbound values", or "free variables"

  slideCB «What do we need to program with names and binders?»

{-
  slideHS «» $
    [$istr|
    |
    |
    |]
-}

  slideO «» $ do
    let pp = p . visibleIncr

    pp !$ «A type for names»
    exampleHS [$str|Name : Set|]

    pp !$ «An equality test for names»
    exampleHS [$str|_≟Name_ : Name → Name → Bool|]

    pp !$ «A generator of fresh names»
    exampleHS [$str|fresh : () → Name|]

    pp !$ «Alternatively: An initial fresh name»
    exampleHS [$str|init : Name|]

    pp !$ «And: How to pick a new one»
    exampleHS [$str|next : Name → Name|]

  BM.frame (BM.ovInts [1,2{-,3-}]) ø [BM.label "dataTm"]
           «Data type for the untyped lambda calculus» ø . vcenter !$? do

    altHS
      [ [$istr|
        |data Tm
        | =  V    Name
        | |  _·_  Tm Tm
        | |  ƛ    Name Tm
        | |  Let  Name Tm Tm
        |]

      , [$istr|
        |data Tm : Set where
        | V    : Name → Tm
        | _·_  : Tm → Tm → Tm
        | ƛ    : Name → Tm → Tm
        | Let  : Name → Tm → Tm → Tm
        |]

      , [$istr|
        |data Tm : Set where
        | V    : (x : Name) → Tm
        | _·_  : (t u : Tm) → Tm
        | ƛ    : (x : Name) (t : Tm) → Tm
        | Let  : (x : Name) (t u : Tm) → Tm
        |]

{-
      , [$istr|
        |data Tm outer
        | =          V    (Name outer)
        | |          _·_  (Tm outer)    (Tm outer)
        | | ∃inner.  ƛ    (Binder outer inner)  (Tm inner)
        | | ∃inner.  Let  (Binder outer inner)  (Tm outer) (Tm inner)
        |]
-}

      , [$istr|
        |data Tm α
        | =        V    (Name α)
        | |        _·_  (Tm α) (Tm α)
        | |  ∃ β.  ƛ    (Binder α β)  (Tm β)
        | |  ∃ β.  Let  (Binder α β)  (Tm α) (Tm β)
        |]

      , [$istr|
        |data Tm α : Set where
        | V    : Name α → Tm α
        | _·_  : Tm α → Tm α → Tm α
        | ƛ    : ∀{β} → Binder α β → Tm β → Tm α
        | Let  : ∀{β} → Binder α β → Tm α → Tm β → Tm α
        |]

      , [$istr|
        |data Tm α : Set where
        | V    : (x : Name α) → Tm α
        | _·_  : (t u : Tm α) → Tm α
        | ƛ    : ∀{β} (x : Binder α β) (t : Tm β) → Tm α
        | Let  : ∀{β} (x : Binder α β) (t : Tm α) (u : Tm β) → Tm α
        |]
      ]

  slideCB $ alt 1 «Nothing more!» «Something less?»

  slide «» $ do
    paraC . hsHuge  $ [$str|(λ x. e) x|]
    paraC . bfHuge  $ «can we compare the two '{B.texttt «x»}'s?»

  slide «» $ do
    paraC . bfHuge $ «Comparison is World Homogeneous»
    paraC . hsLarge $
      [$str|_≟Name_ : ∀ {α} → Name α → Name α → Bool|]

  slide «» $ do
    paraC . bfHuge $ «Names inhabit worlds»

    itemize !$? do
      let itm = item . hsLARGE
      itm $ [$istr|World  : Set|]
      itm $ [$istr|Name   : World → Set|]
      itm $ [$istr|ø      : World|]

    box «Example:» !$ p . hsLARGE $
      [$istr|α : World
            |x : Name α
            |]

  slideC . hshuge $
    [$str|¬nameø : Name ø → ⊥|]

  slide «» $ do
    let wa = (⊕(HS . mathlift BM.alert $ M.sub α))
        wb = (⊕(HS . mathlift BM.alert $ M.sub β))
        x  = hs0 "x"
        y  = hs0 "y"
        e  = HS M.e
    paraC . B.decl B._Huge . mh $
      parens (lam (wb x) (wb e)) · (wa x)

    paraC . visibleI [2,3] . B.decl B._Huge . mh $
      parens (lam (wb x) (wb y)) · (wa y)

    paraC . bfHuge . visibleI [3] $ «We need links between worlds!»

  slide «A type for binders» $ do

    paraC . ttHuge . hs $ [$str|Binder α β : Set|]
    itemizeO !$? do
      item «α outer world / β inner world»
      item «A more precise type for a Name β»
      item . hs $ [$str|nameOfBinder : ∀ {α β} → Binder α β → Name β|]
      item «Using {mh _Binder}, name shadowing is permitted»

  slideCB «Name abstraction as existential quantification»

  put $ BM.againframe (BM.ovInts [2,5]) ø [] "dataTm"

  slideO «Computing the size of a term» $ do

    example $
      invisibleI [1] (hs[$istr|size : ∀ {α} → Tm α → ℕ
                              |]) ⊕
      (hs[$istr|
         |size (V _)        = 1
         |size (t · u)      = 1 + size t + size u
         |size (ƛ _ t)      = 1 + size t
         |size (Let _ t u)  = 1 + size t + size u
         |])

    p !$ BM.pause
    itemizeO !$? mapM_ item
      [ «Polymorphic recursion at work!»
      , «No freshning!»
      , «No shifting!»
      ]

  slide «» $ do
    exampleHS
      [$istr|exportBinder :  ∀ {α β} (x : Binder α β) (y : Name β)
            |                → Maybe (Name α)|]

    let y       = "y"
        x       = "x"
        nOfx    = nameOfBinderN·x

    itemizeO !$? do
      item $ «Compares {mh y} and {mh nOfx}»
      item $ «Performs a type refinement on success»
      item $ «Partial!»
      item $ «Injective!»

  slideCB «What about moving names the other way around? {visibleI [2] $ BM.alert «No!»}»

  -- heqName
{-
> heqName : ∀ {α β} → Binder α β → Name α → Name β → Bool
> heqName x y z with exportBinder x z
> ... | just z' = y ≟Name z'
> ... | nothing = false
-}


  slide «Computing free variables» $ do

    example . visibleI [2,3,4,5] $
      altI [2,3] (hs
      [$istr|
      |rm : Name → List Name → List Name
      |rm _  []                   = []
      |rm x  (y ∷ ys)  with x ≟Name y
      |...             | false    = y ∷ rm x ys
      |...             | true     = rm x ys
      |])

      (hs
      [$istr|
      |rm : ∀ {α β} → Binder α β → List (Name β) → List (Name α)
      |rm _  []                   = []
      |rm x  (y ∷ ys)  with exportBinder x y
      |...             | just y   = y ∷ rm x ys
      |...             | nothing  = rm x ys
      |])

    example . visibleI [1,2,3,4,5] $
      altI [1,2] (hs[$istr|fv : Tm → List Name
                           |])
                  (hs[$istr|fv : ∀ {α} → Tm α → List (Name α)
                           |])
      ⊕(hs
      [$istr|
      |fv (V x)        = [ x ]
      |fv (t · u)      = fv t ++ fv u
      |fv (ƛ x t)      = rm x (fv t)
      |fv (Let x t u)  = fv t ++ rm x (fv u)
      |])

    alertC . bfHuge . visibleI [5] $ «{hs "fv"} enjoys a free theorem!»

{-
  slide «Looking up an environment» $ do

    exampleHS $
      [$istr|
      |data Env β = Empty
      |           | ∃ α. Snoc (β ▹ α) (Env α) (Name β) Ty
      |]

    exampleHS
      [$istr|
      |lookupEnv : ∀ α. Name α → Env α → Ty
      |lookupEnv a (Snoc lnk env b ty)
      |  | a ≡ b    = ty
      |  | otherwise = lookupEnv (cast lnk a) env
      |lookupEnv _ Empty = error "unbound value"
      |]

  slide «Typing a term» $ do

    exampleHS $
      [$istr|
      |typing : ∀ α. Env α → Tm α → Ty
      |typing env (V v)
      |  = lookupEnv v env
      |typing env (Lam lnk a ty t)
      |  = ty `TyArrow` typing (Snoc lnk env a ty) t
      |typing env (Let lnk a t u)
      |  = typing (Snoc lnk env a (typing env t)) u
      |typing env (App t u)
      |  = case typing env t of
      |      from `TyArrow` to | from ≡ typing env u → to
      |      _ → error "ill typed"
      |]
-}

  slideO «Importing names» $ do
    exampleHS $
      [$istr|
      |_⊆_         : Rel World
      |⊆-refl      : ∀ {α} → α ⊆ α
      |⊆-trans     : ∀ {α β γ} → α ⊆ β → β ⊆ γ → α ⊆ γ
      |import⊆     : ∀ {α β} → α ⊆ β → Name α → Name β
      |ø-bottom-⊆  : ∀ {α} → ø ⊆ α
      |]

    visibleenvI [2,3] . B.center . p !$ BI.rawTex
      [$istr|
      |\begin{tikzpicture}
      |    \matrix (m) [matrix of math nodes, row sep=2em, column sep=2em, text height=1.5ex, text depth=0.25ex, ampersand replacement=\&]
      |    { α \& \& γ \\
      |        \& \&   \\
      |      β \& \& δ \\ };
      |    \path[-]
      |    (m-1-3) edge node[above] {\texttt{Binder}} (m-1-1);
      |    \path[-]
      |    (m-1-1) edge node[left] {\begin{turn}{ -90}$⊆$\end{turn}} (m-3-1);
      |    \path[-][dashed]
      |    (m-1-3) edge node[right] {\begin{turn}{ -90}$⊆$\end{turn}} (m-3-3);
      |    \path[-][dashed]
      |    (m-3-3) edge node[below=3pt] {\texttt{Binder}} (m-3-1);
      |\end{tikzpicture}
      |]
    p !$ BM.pause

    exampleHS $
      [$istr|_Binder-commute-⊆_ :  ∀ {α β γ}  → Binder α γ → α ⊆ β
            |                      → ∃ λ δ    → γ ⊆ δ × Binder β δ|]

  -- Strong links vs BINDER
  slideHS «Strong binders»
    [$istr|
    |BINDER    : Rel World
    |weaken    : ∀ {α β} → BINDER α β → Binder α β
    |dropName  : ∀ {α β} → BINDER α β → α ⊆ β
    |]

  slideHS «Generating names»
    [$istr|
    |Fresh       : World → Set
    |Fresh α     = ∃ λ β → BINDER α β
    |freshø      : Fresh ø
    |nextBINDER  : ∀ {α β γ}  → Binder α β → BINDER α γ
    |                         → Fresh β
    |]

  slide «Safety Properties» $ do
    p !$ «Well-typed programs...»
    itemizeO !$? do
      item «... do not get stuck»
      item «... preserve α-equivalence»
      item «... do not break name abstraction»
      item «... do not mix names with different scopes»

  let eqns i0 xs = put (f xs) >> return (i0 + length xs)
        where visi = mathlift . visibleFromInt
              f = B.displaymath . M.array [B.r,B.c,B.l]
                . zipWith (\i (x,y) -> M.cells . map (visi i) $ [x,"≝",y])
                          [i0..]
      -- fp: j'ai mis \iff plutôt que ≝ car ce sont des propositions
      -- np: je change dans l'autre sens

  slideCB «How it works?»

  slide «Nominal implementation» $ do
    let (a,b) = ("a","b")--(M.a,M.b)
    i0 <- eqns 1
      [(unHS $ _World,           unHS $ _Pred·_ℕ)
      ,(unHS $ _Name·β,          unHS $ "{ a : ℕ | "⊕a ∈ β⊕" }")
      ,(unHS $ _Binder·α·β,      "{ a : ℕ | β ≡ {a} ∪ α }")
      ,(unHS $ _BINDER·α·β,      "{ a : ℕ | β ≡ {a} ∪ α ∧ a > α }")
      ,(unHS $ α ⊆ β,            unHS $ "∀ a, "⊕(a ∈ α ⟹ a ∈ β))
      ,(unHS $ eqName_,          unHS $ eqNat_)
      ,(unHS $ exportBinderN·a·b, unHS $ cond (eqNat a b) nothing (just · b))
      ,(unHS $ nextBINDERN·a·b,   unHS $ b ⊔ (a + 1))
      ]

    visibleenvI [i0] . B.para !$
      «{mh importIN}, {mh nameOfBinderN}, {mh weaken},
       {mh dropNameN}, {mh comBinderIncl}, {mh transIN},
       {mh reflI} are computation free.»

  slideCB «Why is this safe?»

  slide «Logical relations recap» $ do
    let (f,g,v,w,x,y,i) = (HS M.f,HS M.g,HS M.v,HS M.w,HS M.x,HS M.y,HS M.i)
        pair l r ix = "("⊕l⊕ix⊕", "⊕r⊕ix⊕")"
    eqns 1
     [(unHS $ logeq f g (τ ⟶ σ),     unHS $ "∀"<+>v<+>w⊕", "⊕(logeq v w τ ⟹
                                         logeq (f · v) (g · w) σ))
     ,(unHS $ idxrefl logeq (pair x y) (τ × σ), unHS $ ilogeq x τ ∧ ilogeq y σ)
--      ,(unHS $ logeq f g "∀ {α} → τ", unHS $ "∀ α, "⊕logeq (f · "{α}") (g · "{α}") τ)
--      ,(unHS $ logeq "(α₁ , x₁)" "(α₂ , x₂)" "∃ λ α → τ", unHS $ "∃ α, "⊕ilogeq x τ)
     ,(unHS $ ilogeq i _ℕ,           unHS $ idxrefl (≡) (i⊕))
     ]

  slide «Worlds as atom bijections» $ do
    descriptionO !$? do
      pure . B.item' «Shadowing extension» !$? do
        B.para !$ «{[$qm|a₁ ((b₁,b₂) ⊞ α) a₂|]} holds if and only if either»
        itemize !$? do
          item «{[$qm|a₁ ≡ b₁ ∧ a₂ ≡ b₂|]} or,»
          item [$qm|a₁ ≢ b₁ ∧ a₂ ≢ b₂ ∧ a₁ (α) a₂|]
      pure . B.item' «Fresh extension» !$? do
        let (b1,b2) = ("b₁","b₂")
            (>) = binop ">"
        B.para !$ «{[$qm|(b₁,b₂)α|]} is»
        itemize !$? do
          item «defined only if {mh$ b1 > dom(α)} and {mh$ b2 > codom(α)}»
          item «equal to {[$qm|{(b₁,b₂)} ∪ α|]} when defined»

  slide «Nominal logical relations» $ do
  -- a₁ (Name β) a₂ ≝ a₁ (β) a₂
    let a = "a" --M.a
    eqns 1
      [(unHS $ ilogeq a (_Name·β),      unHS $ ilogeq a β)
      ,(unHS $ ilogeq a (_Binder·α·β),  unHS $ β ≡ iwover a α)
      ,(unHS $ ilogeq a (_BINDER·α·β),  unHS $ β ≡ "(a₁,a₂) α")
      ,(unHS $ rlogeq unit (α ⊆ β),     unHS $ (α ⊆ β))
      ]

  slide «Soundness theorem for nominal logical relations» $ do
    BM.theorem !$? do
      B.para !$ «Every primitive operation {ma M.p} of type
                 τ is related to itself at type τ»
    {-
    BM.example !$? do
      let a = HS M.a
          b = HS M.b
      B.para !$ «{mh exportBinderN} is sound if {mh (β ≡ iwover a α)} and
                 {mh (ilogeq b β)} hold, then the terms
                 {mh (parens (exportBinderN·"a₁"·"b₁"))} and
                 {mh (parens (exportBinderN·"a₂"·"b₂"))}
                 are related at type {mh (_Maybe · (parens (_Name·α)))}.»
    -}

  todo (return ()) -- "Adequacy with nominal pitt's terms"

  slideCB «Come on, De Bruijn indices are much simpler»

  slide «De Bruijn implementation (types)» $ do
    eqns 1
      [(unHS $ _World,       unHS $ _ℕ)
      ,(unHS $ _Name·β,      mconcat[unHS (_Fin·β)," ⇔ ",M.mleft '[',0,",",β,M.mright ')'])
      ,(unHS $ _Binder·α·β,  unHS $ β ≡ (α + 1))
      ,(unHS $ _BINDER,      unHS $ _Binder)
      ,(unHS $ α ⊆ β,        unHS $ "{ k : ℕ | β ≡ α + k }")
      ]

  slide «De Bruijn implementation (primitives)» $ do
    let i   = HS M.i
        k   = HS M.k
        k'  = HS (M.k⊕"'")
    i0 <- eqns 1
      [(unHS $ eqName_,                  unHS $ eqNat_)
      ,(unHS $ exportBinderD () 0,       unHS $ nothing)
      ,(unHS $ exportBinderD ()
                (parens (suc · i)),      unHS $ just · i)
      ,(unHS $ importID·k·i,             unHS $ k + i)
      ,(unHS $ transID·k·k',             unHS $ k + k')
      ,(unHS $ nameOfBinderD (),         unHS $ 0)
      ,(unHS $ dropNameD·unit,           unHS $ 1)
      ,(unHS $ reflI,                    unHS $ 0)
      ]

    visibleenvI [i0] . B.para !$
      «{mh nextBINDERD}, {mh weaken},
       and {mh comBinderIncl} are computation free.»

  slideO «(well-scoped) De Bruijn indices advantages» $ do
    B.para . bfLarge !$ «Canonical representation»
    B.para !$ BM.pause
    itemizeO !$? do
      item «α-equivalence made simple!»
      item «Structural induction!»
    B.para . visibleI [4..6] . bfLarge !$ «Guaranteed well-formness»
    B.para !$ BM.pause
    itemizeO !$? do
      let n = HS M.n
      item «The {mh$ _Fin·n} technique is well known for that»
      item «Every variable has to be smaller than {mh n}»

  slideCB «It is trivially safe, right? {visibleI [2] $ BM.alert «No!»}»

  slide «De Bruijn indices canonicity: precisely» $ do
    itemizeO !$? do
      item «Bound and fresh names uniquely chosen.»
      item «Free names are freely chosen, though!»
      item «Consequence: α-equivalence is not so simple!»
      item «α-equivalences are not automatically preserved!»

  slide «Worlds as bijections again» $ do
    let k = HS M.k
    descriptionO !$? do
      pure . B.item' «Shifting» !$? do
        B.para . mh !$ wshift α ⊕ " ≝ {(0,0)} ∪ {(i₁+1,i₂+1) | (i₁,i₂) ∈ α}"
      pure . B.item' «Iterated shifting» !$? do
        B.para !$ «We write »⊕mh(wshiftn k α)⊕« for the result of
                   shifting the world α {mh k} times»

  slide «De Bruijn logical relations» $ do
    let (k,i)=(HS M.k,HS M.i)
    eqns 1
      [(unHS $ ilogeq i     (_Name·β),      unHS $ ilogeq i β)
      ,(unHS $ rlogeq unit  (_Binder·α·β),  unHS $ β ≡ wshift α)
      ,(unHS $ rlogeq unit  (_BINDER·α·β),  unHS $ β ≡ wshift α)
      ,(unHS $ rlogeq k     (α ⊆ β),        unHS $ β ≡ wshiftn k α)
      ]

  slide «Soundness theorem for De Bruijn logical relations» $ do
    BM.theorem !$? do
      B.para !$ «Every primitive operation {ma M.p} of type
                 τ is related to itself at type τ»
    {-
    BM.example !$? do
      let i :: HS Integer
          i = HS M.i
      B.para !$ «{mh exportBinderD_} is sound if {mh (β ≡ wshift α)} and
                 {mh (ilogeq i β)} hold, then the terms
                 {mh (parens (exportBinderD () "i₁"))} and {mh (parens (exportBinderD () "i₂"))}
                 are related at type {mh $ _Maybe · (parens (_Name·α))}.»
    -}

  slide «Listings of some advanced examples» $ do
    itemizeO !$? do
      item «Deciding α-equivalence on types like {mh _Tm}»
      item «Generic export through a binder»
      item «Generic import through a world inclusion witness»
      item «Normalisation by evaluation»

{-
  section «Status about the formalization»
    single abs, but arbitrary binding scope
    no ADTs sum/product
    some more typing rules
    reduction fails dynamically on casts, create, and fresh
    aeq
    thm
      item «{purefreshml} = this + single world»
-}

  slide «Challenges and future work» $ do
    itemizeO !$? do
      item «More powerful logical relations (relating to an abstract model)»
      item «How to carry information on binders (types, specs...)»
      item «More formalisation of the logical relations»
      item «More advanced usages»

  slide «Conclusion» $ do
    itemizeO !$? do
      item «Explicit scopes using world indices»
      item «Non-freshening opening»
      item «Name abstraction as existential quantification»
      item «Expressiveness close to a manual model with names»
      item «Two concrete and safe implementations of the interface»

  slideCB «Questions?»

  put BM.appendix

  let go trg titl = B.para $ BM.hyperlink [] trg (BM.beamerbutton titl) []
  let gobackbonus =
         B.para !$ BM.hyperlink [] "bonus" (BM.beamerreturnbutton «Back») []
  slideL «Bonus track» "bonus" !$? do
    go "occurs" . hs !$ "occurs"
    go "star" . hs !$ "Star"
    go "env" . hs !$ "★Binder"
    go "exportStar" . hs !$ "export★Binder"
    go "fv'" . hs !$ "fv'"
    go "pack" !$ «Packaging»
    go "app" . hs !$ "app"
    go "context" !$ «Contexts»
    go "patt" !$ «Patterns»
    go "nameEqH" . hs !$ "nameEqH"

  -- put $ BM.againframe ø ø [] "dataTm"
  put $ BM.againframe (BM.ovInts [1..6]) ø [] "dataTm"

  -- slideCB «Polymorphic values represent closed terms»
  -- * forall alpha. Tm alpha
  --   there cannot be any free atom in it
  -- * forall alpha. Tm alpha -> Tm alpha OR Name alpha
  --   free atoms of the output are necessary a subset of input free atoms

  -- slideCB «A more direct presentation of atom sorts»
  -- atom sorts are typically added afterward for pragmatic
  -- reasons, however they add some complexity to the presentation
  -- without adding any theoretical benefit.

  -- slideCB «Generalizing {alphaCaml} data structures»
  -- * alphaCaml has two different kind of types,
  --   pattern types and expression types.
  -- * Pattern types describe the shape of binders.
  -- * Abstraction pack pattern types and expression
  --   types as a single abstract representation
  --   supporting alpha equivalence
  -- * These distinctions where guided by practical
  --   use cases rather than some uniform presentation.

  {-
  slideCB «Reaching {ott} expressiveness and more»
  -- * Ott enables to define the set of bound atoms,
  --   and also where they are bound in a very general way.
  -- * This gives a more general presentation that patt/expr types
  -- * One would like to go further by allowing to dynamically
  --   select bound atoms.
  -}

--   slide «Precise control over scopes» $ do
--     box «Having explicit world subsume:» !$
--       itemizeO $? do
--         item «{alphaCaml} inner/outer/neutral annotations»
--         item «{alphaCaml} pattern types/expression types distinction»
--         item «{freshml}/{alphaCaml} atom sorts»

  slideL «Occurs check» "occurs" !$? do

    exampleHS
      [$istr|
      |occurs : ∀ {α} → Name α → Tm α → Bool
      |occurs x₀ = occ (λ y → x₀ ≟Name y)
      |  where
      |    OccEnv  : World → Set
      |    OccEnv  α = Name α → Bool
      |    extend  : ∀ {α β} → Binder α β → OccEnv α → OccEnv β
      |    extend  x Γ y = maybe Γ false (exportBinder x y)
      |
      |    occ : ∀ {α} → OccEnv α → Tm α → Bool
      |    occ Γ  (V y)        = Γ y
      |    occ Γ  (t · u)      = occ Γ t ∨ occ Γ u
      |    occ Γ  (ƛ x t)      = occ (extend x Γ) t
      |    occ Γ  (Let x t u)  = occ Γ t ∨ occ (extend x Γ) u
      |]
    gobackbonus

  slideL «Reflexive and transitive closure of a relation» "star" !$? do

    exampleHS
      [$str|Rel A = A → A → Set|]

    exampleHS
      [$istr|
      |data Star {I : Set} (T : Rel I) : Rel I where
      |  ε    : ∀ {i} → Star T i i
      |  _◅_  : ∀ {i j k} → T i j → Star T j k → Star T i k
      |]

    exampleHS
      [$istr|
      |Star⁻¹  : ∀ {I : Set} → Rel I → Rel I
      |Star⁻¹  = flip ∘ Star ∘ flip
      |]
    gobackbonus

  slideL «Environments as a chain of binders» "env" !$? do

    exampleHS
      [$istr|
      |★Binder  : Rel World
      |★Binder  = Star⁻¹ Binder
      |]

    exampleHS
      [$istr|
      |  ε  : ∀ {α} → ★Binder α α
      |_◅_  : ∀ {α β γ}  (x : Binder β γ) (Γ : ★Binder α β)
      |               →  ★Binder α γ
      |]
    gobackbonus

  slideL «Exporting through a chain of binders» "exportStar" !$? do

    exampleHS
      [$istr|
      |export★Binder  : ∀ {α β}  → ★Binder α β
      |                          → Name β → Maybe (Name α)
      |export★Binder  ε        y
      |  = just y
      |export★Binder  (x ◅ Γ)  y
      |  = exportBinder x y >>= export★Binder Γ
      |  where open MaybeMonad
      |]
    gobackbonus

  slideL «Free variables, continued...» "fv'" !$? do

    exampleHS
      [$istr|
      |fv'  : ∀ {β α} → ★Binder α β → Tm β → List (Name α)
      |fv'  Γ  (V x)        = List.fromMaybe (export★Binder Γ x)
      |fv'  Γ  (t · u)      = fv' Γ t ++ fv' Γ u
      |fv'  Γ  (ƛ x t)      = fv' (x ◅ Γ) t
      |fv'  Γ  (Let x t u)  = fv' Γ t ++ fv' (x ◅ Γ) u
      |]
    gobackbonus

  slideL «Packaging up...» "pack" !$? do
    itemize !$? do
      let itm = item . hs
      itm "binderOf"
      itm "nameOf"
      itm "nextOf"
      itm "importWith"
    gobackbonus

  slideL «Constructing terms» "app" !$? do
    exampleHS
      [$istr|
      |app  : Tm ø
      |app  = ƛ  (binderOf x) (ƛ (binderOf y)
      |          (V (importWith y (nameOf x)) · V (nameOf y)))
      |  where  open FreshPack
      |         x  = freshø
      |         y  = nextOf x
      |]
    gobackbonus

  slideL «Towards elaborate uses of worlds» "context" !$? do
    exampleHS
      [$istr|
      |data C α : (β : World) → Set where
      |  Hole  : C α α
      |  _·₁_  : ∀ {β} (c : C α β) (t : Tm α) → C α β
      |  _·₂_  : ∀ {β} (t : Tm α) (c : C α β) → C α β
      |  ƛ     : ∀ {β γ}  (x : Binder α β) (τ : Ty)
      |                   (c : C β γ) → C α γ
      |  Let₁  : ∀ {β γ}  (x : Binder α β) (c : C α γ)
      |                   (t : Tm β) → C α γ
      |  Let₂  : ∀ {β γ}  (x : Binder α β) (t : Tm α)
      |                   (c : C β γ) → C α γ
      |]
    gobackbonus

  slideL «Patterns types» "patt" !$? do
    exampleHS
      [$istr|
      |data Pa α β : Set where
      |  PVar   : Binder α β → Pa α β
      |  PPair  : ∀{γ} → Pa α γ → Pa γ β → Pa α β
      |]
    gobackbonus

--   slideCB «Safe heterogeneous comparison!»

  slideL «Safe heterogeneous comparison!» "nameEqH" !$? do
    exampleHS [$str|nameEqH : ∀{α β}→ Name α → Name β → Binder α β → Bool|]
    example $ hs [$istr|nameEqH x y lnk with exportBinder lnk y
                       |...                | just y' = x ≡ y'
                       |...                | nothing = false|]
    gobackbonus

{-
  slide «Substituting closed terms for variables» $ do
    alertC «Outdated»
    exampleHS $
      [$istr|
      |substClosed :
      |    ∀ α. Atm α → (∀ β. Tm β) → Tm α → Tm α
      |substClosed a v = go id
      |  where
      |    go : ∀ δ. (δ ▹ α) → Tm δ → Tm δ
      |    go lnk (V b) | atmEqH a b lnk = v
      |                   | otherwise      = V b
      |    go lnk (App t u)
      |      = App (go lnk t) (go lnk u)
      |    go lnk (Lam lnk' b ty t)
      |      = Lam lnk' b ty (go (lnk ∘ lnk') t)
      |    go lnk (Let lnk' b t u)
      |      = Let lnk' b (go lnk t) (go (lnk ∘ lnk') u)|]
      -- |    go _   (Cst cst)         = Cst cst
-}

